/////////////////////////////////////////////////////////////////////////////
// This file is part of the "Java-DAP" project, a Java implementation
// of the OPeNDAP Data Access Protocol.
//
// Copyright (c) 2010, OPeNDAP, Inc.
// Copyright (c) 2002,2003 OPeNDAP, Inc.
// 
// Author: James Gallagher <jgallagher@opendap.org>
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms,
// with or without modification, are permitted provided
// that the following conditions are met:
// 
// - Redistributions of source code must retain the above copyright
//   notice, this list of conditions and the following disclaimer.
// 
// - Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the
//   documentation and/or other materials provided with the distribution.
// 
// - Neither the name of the OPeNDAP nor the names of its contributors may
//   be used to endorse or promote products derived from this software
//   without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/////////////////////////////////////////////////////////////////////////////


package opendap.servlet;


import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Default handler for OPeNDAP directory requests. This class is used
 * by AbstractServlet. This code exists as a seperate class in order to alleviate
 * code bloat in the AbstractServlet class. As such, it contains virtually no
 * state, just behaviors.
 *
 * @author Nathan David Potter
 */

public class GetDirHandler {

    private static final boolean _Debug = false;
    private String separator;




    public GetDirHandler() {
        setSeparator("/");
    }

    public String getSeparator(){
        return separator;
    }

    public void setSeparator(String sep){
        separator = sep;
    }



    /**
     * ************************************************************************
     * Default handler for OPeNDAP directory requests. Returns an html document
     * with a list of all datasets on this server with links to their
     * DDS, DAS, Information, and HTML responses.
     *
     * @param request  The <code>HttpServletRequest</code> from the client.
     * @param response The <code>HttpServletResponse</code> for the client.
     * @param rs       The request state object for this client request.
     * @see ReqState
     */
    public void sendDIR(HttpServletRequest request,
                        HttpServletResponse response,
                        ReqState rs)  {

        if (_Debug) System.out.println("sendDIR request = " + request);

        String ddxCacheDir = rs.getDDXCache();
        String ddsCacheDir = rs.getDDSCache();

        try {

            PrintWriter pw = new PrintWriter(response.getOutputStream());

            String thisServer = request.getRequestURL().toString();
            pw.println("<html>");
            pw.println("<head>");
            pw.println("<title>OPeNDAP Directory</title>");
            pw.println("<meta http-equiv=\"Content-Type\" content=\"text/html\">");
            pw.println("</head>");

            pw.println("<body bgcolor=\"#FFFFFF\">");


            pw.println("<h1>OPeNDAP Directory for:</h1>");
            pw.println("<h2>" + Scrub.completeURL(thisServer) + "</h2>");

            pw.println(printDIR(ddxCacheDir, "DDX", thisServer));

            pw.println(printDIR(ddsCacheDir, "DDS", thisServer));
            pw.println("<hr>");
            pw.println("</html>");
            pw.flush();

        } catch (FileNotFoundException fnfe) {
            System.out.println("OUCH! FileNotFoundException: " + fnfe.getMessage());
            fnfe.printStackTrace(System.out);
        } catch (IOException ioe) {
            System.out.println("OUCH! IOException: " + ioe.getMessage());
            ioe.printStackTrace(System.out);
        }


    }



    private String printDIR(String dirName, String dirType, String thisServer) {
        
        StringBuilder pw = new StringBuilder();

        pw.append("<hr />\n");
        pw.append("<h3>").append(dirType).append("</h3>\n");

        File dir = new File(dirName);

        if (dir.exists()) {

            if (dir.isDirectory()) {

                if (_Debug)
                    System.out.append("lastIndexOf(").append(getSeparator()).append("): ").append(String.valueOf(thisServer.lastIndexOf(getSeparator()))).append("\n");
                if (_Debug) System.out.append("length: ").append(String.valueOf(thisServer.length())).append("\n");

                if (thisServer.lastIndexOf(getSeparator()) != (thisServer.length() - 1)) {
                    if (_Debug) System.out.append("Server URL does not end with: ").append(getSeparator()).append("\n");
                    thisServer += getSeparator();
                } else {
                    if (_Debug) System.out.append("Server URL ends with: ").append(getSeparator()).append("\n");
                }


                File fList[] = dir.listFiles();

                pw.append("<table border=\"0\">\n");

                for (File file : fList) {
                    if (file.isFile()) {
                        String fName = file.getName();

                        pw.append("<tr>\n");

                        pw.append("    <td>");
                        pw.append("<div align='right'>");
                        pw.append("<b>").append(fName).append(":</b> ");
                        pw.append("</div>");
                        pw.append("</td>\n");


                        pw.append("    <td>");
                        pw.append("<div align='center'>");
                        pw.append("<a href='").append(thisServer).append(fName).append(".ddx'> DDX </a>");
                        pw.append("</div>");
                        pw.append("</td>\n");

                        pw.append("    <td>");
                        pw.append("<div align='center'>");
                        pw.append("<a href='").append(thisServer).append(fName).append(".dds'> DDS </a>");
                        pw.append("</div>");
                        pw.append("</td>\n");

                        pw.append("    <td>");
                        pw.append("<div align='center'>");
                        pw.append("<a href='").append(thisServer).append(fName).append(".das'> DAS </a>");
                        pw.append("</div>");
                        pw.append("</td>\n");

                        pw.append("    <td>");
                        pw.append("<div align='center'>");
                        pw.append("<a href='").append(thisServer).append(fName).append(".info'> Information </a>");
                        pw.append("</div>");
                        pw.append("</td>\n");

                        pw.append("    <td>");
                        pw.append("<div align='center'>");
                        pw.append("<a href='").append(thisServer).append(fName).append(".html'> HTML Data Request Form </a>");
                        pw.append("</div>");
                        pw.append("</td>\n");

                        pw.append("</tr>\n");
                    }
                }
                pw.append("</table>\n");

            } else {
                pw.append("<h3>\n");
                pw.append("Specified ").append(dirType).append(" cache:<br>\n");
                pw.append("<i>").append(dirName).append("</i><br>\n");
                pw.append("is not a directory!\n");
                pw.append("</h3>\n");
            }
        } else {
            pw.append("<h4>\n");
            pw.append("Cannot Find ").append(dirType).append(" Directory:<br>\n");
            pw.append("<i>").append(dirName).append("</i><br>\n");
            pw.append("</h4>\n");
        }


        return pw.toString();

    }
    

}





